-- Reference for saved variables:
--		Default values are shown when applicable

SKG_DBPC = {
------------------- Players -------------------
	--
	-- Players is the structure that contains the mapping of usernames to IDs
	-- If the "AssociateAlts" method is used (see security options), then the
	-- "Alts" table will ALSO exist. Entries are only made in the "Alts" table
	-- when a new toon is added as an alt. It will NOT have an entry when the
	-- record with a given ID is a main.
	--
	-- The ID is a simple hash of the character's name. See
	-- SKG_UtilityFunctions\SKG_HashName and will always be 24 characters long.
	-- For example: EAD908D2098BF9D809398CD0"
	--
	Players = {
		ID = {
			name = "ToonName",
			class = "ToonClass",
		}
	},
	Alts = {
		ID = {
			[1] = {
				name = "AltName",
				class = "AltClass"
			},
			[2] = {
				name = "SecontAltName",
				class = "SecondAltClass"
			},
			...
		},
		ID = {
			...
		},
		...
	}
------------------ RollLists ------------------
	--
	-- RollLists is the structure containing the actual SKG Toon lists
	-- There is currently only one "disallowed" Listname, that being "SKG"
	-- which is used for administrative purposes
	--
	-- the LastMod table stores the last time that particular PID entries
	-- in each RollList were modified in some significant way, usually through
	-- a loot event. "Manual" list operations (eg, move up/down) do NOT change
	-- the LastMod timestamp.
	RollLists = {
		"ListName" = {
			[1] = ID,
			[2] = ID,
			...
		},
		"Second List" = {
		},
	},
	LastMod = {
		"ListName" = {
			ID = lastMod,
			ID = lastMod,
			...
		},
		"Second List" = {
		},
		...
	},
------------------ Snapshot ------------------
	--
	-- The "Snapshot" structure is used to make a "saved copy" of a list when a snapshot operation
	-- is performed. Snapshotting resets the transaction log for the list in question, reducing it
	-- to a single authoritative entry. The saved copy in this structure is identical in structure
	-- to the "RollList" structure above.
	--
	Snapshot = {
		["ListName"] = { ... },
		["Second List"] = {...}
	},
------------------ Log ------------------
	-- The "Log" structure is a transaction log for the given lists. Each transaction that affects
	-- the position of any node in the list (or node creation/deletion) has a log entry. The first
	-- even in any log will always be a "SYNSS" (Snapshot) event, either from initial list creation
	-- or an actual snapshot event.
	--
	-- Log entries in the special reserved "SKG" list will be used for the following purposes:
	--		- Record all list operations (creation, delete, rename, clone)
	--		- Record all "player" transaction entries (add, remove, rename)
	--		- Record all "alt" player transactions (add-alt, remove-alt, rename-alt)
	--
	-- Contents of a log node are:
	--		TID			- transaction ID. See Documentation/GeneralNotes.lua for TID format
	--			token	- See Documentation/EventTokens.lua for details on the various events
	--			payload	- required data to process the above token. Varies per token
	--			nlHash	- New List Hash - hash of the resultant list AFTER the token has been processed
	--
	Log = {
		["SKG"] = {
		},
		["ListName"] = {
			[TID] = {
				token = "CMDTK",
				payload = "",
				nlHash = "longListHash"
			}
		},
		["Second List"] = {...},
	},
------------------ Options ------------------
	--
	-- This structure contains all the options for the various
	-- aspects of the GUI and plugin config. Some options are
	-- only available for users of sufficient security level
	--
	Options = {
		--
		-- General options:
		-- xPos				- Saved x position of the top left corner of the main frame
		-- yPos				- Saved y position of hte top left corner of the main frame
		-- buttonTexture	- selected button texture. Currently valid: "1" through "7"
		-- classColors		- shows class colors in list? (true/false)
		-- raidonly			- show only current raid members? (true/false)
		-- 
		["General"] = {
			xPos = 100,
			yPos = -100,
			buttonTexture = 1,
			classColors = true,
			raidonly = false
		},
		--
		-- Security options:
		-- RankBasedAuth	- rank based authentication? (true/false)
		-- AuthRank			- which rank (and above) has privileges? (rank index)
		--					-- only used if "RankBasedAuth" is true
		-- ManualAuthList	- List if players who have privileges
		--					-- only used if "RankBasedAuth" is false
		-- UseProxy			- Is Proxy admin enabled? (true/false)
		-- Proxy			- Assigned Security Proxy
		--					-- only used if "UseProxy" is true
		--
		["Security"] = {
			RankBasedAuth = true,
			AuthRank = 1,
			ManualAuthList = {
				Playername, ...
			},
			UseProxy = false,
			Proxy = "",
		},
		--
		-- List options:
		-- (tbd)
		-- 
		["List"] = {
		},
		--
		-- Bid options:
		-- ArmorFilter	- Armor Filtering value (valid: 0, 1, 2)
		-- RankFilter	- not currently implemented
		-- ClassFilter	- bit string of class filtering (9 digits, 0 or 1)
		--
		["Bid"] = {
			ArmorFilter = 1,
			RankFilteer = 0,
			ClassFilter = "111111111",
		}
	}
}
